# /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# INSTITUTO TECNOLOGICO DE COSTA RICA
# Escuela de Ingenieria en Construccion
# Construction Engineering School
# https://www.tec.ac.cr
# Eng. MSc. Maikel Mendez Morales
# Email: maikel.mendez@gmail.com; mamendez@itcr.ac.cr
# https://orcid.org/0000-0003-1919-141X
# https://www.scopus.com/authid/detail.uri?authorId=51665581300
# https://scholar.google.com/citations?user=JnmSVFYAAAAJ&hl=en
# https://www.youtube.com/c/maikelmendez
# https://twitter.com/MaikelMendezM
# https://github.com/maikelonu
# Skype: maikel.mendez
# /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# INFO: This script is intended for the execution of raster-based bias-corrected data by {R-downscaleR}, including 
# gamma-pareto quantile mapping (GPQM),  mean-variance adjustment (DELTA), Power transformation of precipitation (PTR) 
# for the period 1981-1995 (15 years)
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# MANUSCRIPT TITLE:
# "Performance Evaluation of Bias Correction Methods for Monthly Precipitation 
#  Projections over Costa Rica"
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# INPUT FILES:

# netCDF format: all *.nc files included in the following lists

# 1951-1980 period **
# hist_1951_1980_IMN.nc
# hist_1951_1980_GPCC.nc
# hist_1951_1980_PRECIS.nc
# hist_1951_1980_CanESM2.nc
# hist_1951_1980_MIROC.nc
# hist_1951_1980_EARTH.nc
# hist_1951_1980_HadGEM2.nc
# hist_1951_1980_ENSAMBLE.nc

# 1981-1995 period **
# hist_1981_1995_IMN.nc
# hist_1981_1995_GPCC.nc
# hist_1981_1995_PRECIS.nc
# hist_1981_1995_CanESM2.nc
# hist_1981_1995_MIROC.nc
# hist_1981_1995_EARTH.nc
# hist_1981_1995_HadGEM2.nc
# hist_1981_1995_ENSAMBLE.nc
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# MANUSCRIPT FIGURES:
# None
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# OUTPUT FILES:
# Individual GeoTIFF files as listed per month:

# PTR **
# GPCC_1981_1995_PTR_BC_JAN.tiff,	GPCC_1981_1995_PTR_BC_FEB.tiff,	GPCC_1981_1995_PTR_BC_MAR.tiff,	GPCC_1981_1995_PTR_BC_APR.tiff,	GPCC_1981_1995_PTR_BC_MAY.tiff,	GPCC_1981_1995_PTR_BC_JUN.tiff,	GPCC_1981_1995_PTR_BC_JUL.tiff,	GPCC_1981_1995_PTR_BC_AUG.tiff,	GPCC_1981_1995_PTR_BC_SEP.tiff,	GPCC_1981_1995_PTR_BC_OCT.tiff,	GPCC_1981_1995_PTR_BC_NOV.tiff,	GPCC_1981_1995_PTR_BC_DEC.tiff,
# PRECIS_1981_1995_PTR_BC_JAN.tiff,	PRECIS_1981_1995_PTR_BC_FEB.tiff,	PRECIS_1981_1995_PTR_BC_MAR.tiff,	PRECIS_1981_1995_PTR_BC_APR.tiff,	PRECIS_1981_1995_PTR_BC_MAY.tiff,	PRECIS_1981_1995_PTR_BC_JUN.tiff,	PRECIS_1981_1995_PTR_BC_JUL.tiff,	PRECIS_1981_1995_PTR_BC_AUG.tiff,	PRECIS_1981_1995_PTR_BC_SEP.tiff,	PRECIS_1981_1995_PTR_BC_OCT.tiff,	PRECIS_1981_1995_PTR_BC_NOV.tiff,	PRECIS_1981_1995_PTR_BC_DEC.tiff,
# CanESM2_1981_1995_PTR.BC_JAN.tiff,	CanESM2_1981_1995_PTR.BC_FEB.tiff,	CanESM2_1981_1995_PTR.BC_MAR.tiff,	CanESM2_1981_1995_PTR.BC_APR.tiff,	CanESM2_1981_1995_PTR.BC_MAY.tiff,	CanESM2_1981_1995_PTR.BC_JUN.tiff,	CanESM2_1981_1995_PTR.BC_JUL.tiff,	CanESM2_1981_1995_PTR.BC_AUG.tiff,	CanESM2_1981_1995_PTR.BC_SEP.tiff,	CanESM2_1981_1995_PTR.BC_OCT.tiff,	CanESM2_1981_1995_PTR.BC_NOV.tiff,	CanESM2_1981_1995_PTR.BC_DEC.tiff,
# MIROC_1981_1995_PTR_BC_JAN.tiff,	MIROC_1981_1995_PTR_BC_FEB.tiff,	MIROC_1981_1995_PTR_BC_MAR.tiff,	MIROC_1981_1995_PTR_BC_APR.tiff,	MIROC_1981_1995_PTR_BC_MAY.tiff,	MIROC_1981_1995_PTR_BC_JUN.tiff,	MIROC_1981_1995_PTR_BC_JUL.tiff,	MIROC_1981_1995_PTR_BC_AUG.tiff,	MIROC_1981_1995_PTR_BC_SEP.tiff,	MIROC_1981_1995_PTR_BC_OCT.tiff,	MIROC_1981_1995_PTR_BC_NOV.tiff,	MIROC_1981_1995_PTR_BC_DEC.tiff,
# EARTH_1981_1995_PTR_BC_JAN.tiff,	EARTH_1981_1995_PTR_BC_FEB.tiff,	EARTH_1981_1995_PTR_BC_MAR.tiff,	EARTH_1981_1995_PTR_BC_APR.tiff,	EARTH_1981_1995_PTR_BC_MAY.tiff,	EARTH_1981_1995_PTR_BC_JUN.tiff,	EARTH_1981_1995_PTR_BC_JUL.tiff,	EARTH_1981_1995_PTR_BC_AUG.tiff,	EARTH_1981_1995_PTR_BC_SEP.tiff,	EARTH_1981_1995_PTR_BC_OCT.tiff,	EARTH_1981_1995_PTR_BC_NOV.tiff,	EARTH_1981_1995_PTR_BC_DEC.tiff,
# HadGEM2_1981_1995_PTR_BC_JAN.tiff,	HadGEM2_1981_1995_PTR_BC_FEB.tiff,	HadGEM2_1981_1995_PTR_BC_MAR.tiff,	HadGEM2_1981_1995_PTR_BC_APR.tiff,	HadGEM2_1981_1995_PTR_BC_MAY.tiff,	HadGEM2_1981_1995_PTR_BC_JUN.tiff,	HadGEM2_1981_1995_PTR_BC_JUL.tiff,	HadGEM2_1981_1995_PTR_BC_AUG.tiff,	HadGEM2_1981_1995_PTR_BC_SEP.tiff,	HadGEM2_1981_1995_PTR_BC_OCT.tiff,	HadGEM2_1981_1995_PTR_BC_NOV.tiff,	HadGEM2_1981_1995_PTR_BC_DEC.tiff,
# ENSAMBLE_1981_1995_PTR_BC_JAN.tiff,	ENSAMBLE_1981_1995_PTR_BC_FEB.tiff,	ENSAMBLE_1981_1995_PTR_BC_MAR.tiff,	ENSAMBLE_1981_1995_PTR_BC_APR.tiff,	ENSAMBLE_1981_1995_PTR_BC_MAY.tiff,	ENSAMBLE_1981_1995_PTR_BC_JUN.tiff,	ENSAMBLE_1981_1995_PTR_BC_JUL.tiff,	ENSAMBLE_1981_1995_PTR_BC_AUG.tiff,	ENSAMBLE_1981_1995_PTR_BC_SEP.tiff,	ENSAMBLE_1981_1995_PTR_BC_OCT.tiff,	ENSAMBLE_1981_1995_PTR_BC_NOV.tiff,	ENSAMBLE_1981_1995_PTR_BC_DEC.tiff,

# GPQM **
# GPCC_1981_1995_GPQM_BC_JAN.tiff,	GPCC_1981_1995_GPQM_BC_FEB.tiff,	GPCC_1981_1995_GPQM_BC_MAR.tiff,	GPCC_1981_1995_GPQM_BC_APR.tiff,	GPCC_1981_1995_GPQM_BC_MAY.tiff,	GPCC_1981_1995_GPQM_BC_JUN.tiff,	GPCC_1981_1995_GPQM_BC_JUL.tiff,	GPCC_1981_1995_GPQM_BC_AUG.tiff,	GPCC_1981_1995_GPQM_BC_SEP.tiff,	GPCC_1981_1995_GPQM_BC_OCT.tiff,	GPCC_1981_1995_GPQM_BC_NOV.tiff,	GPCC_1981_1995_GPQM_BC_DEC.tiff,
# PRECIS_1981_1995_GPQM_BC_JAN.tiff,	PRECIS_1981_1995_GPQM_BC_FEB.tiff,	PRECIS_1981_1995_GPQM_BC_MAR.tiff,	PRECIS_1981_1995_GPQM_BC_APR.tiff,	PRECIS_1981_1995_GPQM_BC_MAY.tiff,	PRECIS_1981_1995_GPQM_BC_JUN.tiff,	PRECIS_1981_1995_GPQM_BC_JUL.tiff,	PRECIS_1981_1995_GPQM_BC_AUG.tiff,	PRECIS_1981_1995_GPQM_BC_SEP.tiff,	PRECIS_1981_1995_GPQM_BC_OCT.tiff,	PRECIS_1981_1995_GPQM_BC_NOV.tiff,	PRECIS_1981_1995_GPQM_BC_DEC.tiff,
# CanESM2_1981_1995_GPQM.BC_JAN.tiff,	CanESM2_1981_1995_GPQM.BC_FEB.tiff,	CanESM2_1981_1995_GPQM.BC_MAR.tiff,	CanESM2_1981_1995_GPQM.BC_APR.tiff,	CanESM2_1981_1995_GPQM.BC_MAY.tiff,	CanESM2_1981_1995_GPQM.BC_JUN.tiff,	CanESM2_1981_1995_GPQM.BC_JUL.tiff,	CanESM2_1981_1995_GPQM.BC_AUG.tiff,	CanESM2_1981_1995_GPQM.BC_SEP.tiff,	CanESM2_1981_1995_GPQM.BC_OCT.tiff,	CanESM2_1981_1995_GPQM.BC_NOV.tiff,	CanESM2_1981_1995_GPQM.BC_DEC.tiff,
# MIROC_1981_1995_GPQM_BC_JAN.tiff,	MIROC_1981_1995_GPQM_BC_FEB.tiff,	MIROC_1981_1995_GPQM_BC_MAR.tiff,	MIROC_1981_1995_GPQM_BC_APR.tiff,	MIROC_1981_1995_GPQM_BC_MAY.tiff,	MIROC_1981_1995_GPQM_BC_JUN.tiff,	MIROC_1981_1995_GPQM_BC_JUL.tiff,	MIROC_1981_1995_GPQM_BC_AUG.tiff,	MIROC_1981_1995_GPQM_BC_SEP.tiff,	MIROC_1981_1995_GPQM_BC_OCT.tiff,	MIROC_1981_1995_GPQM_BC_NOV.tiff,	MIROC_1981_1995_GPQM_BC_DEC.tiff,
# EARTH_1981_1995_GPQM_BC_JAN.tiff,	EARTH_1981_1995_GPQM_BC_FEB.tiff,	EARTH_1981_1995_GPQM_BC_MAR.tiff,	EARTH_1981_1995_GPQM_BC_APR.tiff,	EARTH_1981_1995_GPQM_BC_MAY.tiff,	EARTH_1981_1995_GPQM_BC_JUN.tiff,	EARTH_1981_1995_GPQM_BC_JUL.tiff,	EARTH_1981_1995_GPQM_BC_AUG.tiff,	EARTH_1981_1995_GPQM_BC_SEP.tiff,	EARTH_1981_1995_GPQM_BC_OCT.tiff,	EARTH_1981_1995_GPQM_BC_NOV.tiff,	EARTH_1981_1995_GPQM_BC_DEC.tiff,
# HadGEM2_1981_1995_GPQM_BC_JAN.tiff,	HadGEM2_1981_1995_GPQM_BC_FEB.tiff,	HadGEM2_1981_1995_GPQM_BC_MAR.tiff,	HadGEM2_1981_1995_GPQM_BC_APR.tiff,	HadGEM2_1981_1995_GPQM_BC_MAY.tiff,	HadGEM2_1981_1995_GPQM_BC_JUN.tiff,	HadGEM2_1981_1995_GPQM_BC_JUL.tiff,	HadGEM2_1981_1995_GPQM_BC_AUG.tiff,	HadGEM2_1981_1995_GPQM_BC_SEP.tiff,	HadGEM2_1981_1995_GPQM_BC_OCT.tiff,	HadGEM2_1981_1995_GPQM_BC_NOV.tiff,	HadGEM2_1981_1995_GPQM_BC_DEC.tiff,
# ENSAMBLE_1981_1995_GPQM_BC_JAN.tiff,	ENSAMBLE_1981_1995_GPQM_BC_FEB.tiff,	ENSAMBLE_1981_1995_GPQM_BC_MAR.tiff,	ENSAMBLE_1981_1995_GPQM_BC_APR.tiff,	ENSAMBLE_1981_1995_GPQM_BC_MAY.tiff,	ENSAMBLE_1981_1995_GPQM_BC_JUN.tiff,	ENSAMBLE_1981_1995_GPQM_BC_JUL.tiff,	ENSAMBLE_1981_1995_GPQM_BC_AUG.tiff,	ENSAMBLE_1981_1995_GPQM_BC_SEP.tiff,	ENSAMBLE_1981_1995_GPQM_BC_OCT.tiff,	ENSAMBLE_1981_1995_GPQM_BC_NOV.tiff,	ENSAMBLE_1981_1995_GPQM_BC_DEC.tiff,

# DELTA **
# GPCC_1981_1995_DELTA_BC_JAN.tiff,	GPCC_1981_1995_DELTA_BC_FEB.tiff,	GPCC_1981_1995_DELTA_BC_MAR.tiff,	GPCC_1981_1995_DELTA_BC_APR.tiff,	GPCC_1981_1995_DELTA_BC_MAY.tiff,	GPCC_1981_1995_DELTA_BC_JUN.tiff,	GPCC_1981_1995_DELTA_BC_JUL.tiff,	GPCC_1981_1995_DELTA_BC_AUG.tiff,	GPCC_1981_1995_DELTA_BC_SEP.tiff,	GPCC_1981_1995_DELTA_BC_OCT.tiff,	GPCC_1981_1995_DELTA_BC_NOV.tiff,	GPCC_1981_1995_DELTA_BC_DEC.tiff,
# PRECIS_1981_1995_DELTA_BC_JAN.tiff,	PRECIS_1981_1995_DELTA_BC_FEB.tiff,	PRECIS_1981_1995_DELTA_BC_MAR.tiff,	PRECIS_1981_1995_DELTA_BC_APR.tiff,	PRECIS_1981_1995_DELTA_BC_MAY.tiff,	PRECIS_1981_1995_DELTA_BC_JUN.tiff,	PRECIS_1981_1995_DELTA_BC_JUL.tiff,	PRECIS_1981_1995_DELTA_BC_AUG.tiff,	PRECIS_1981_1995_DELTA_BC_SEP.tiff,	PRECIS_1981_1995_DELTA_BC_OCT.tiff,	PRECIS_1981_1995_DELTA_BC_NOV.tiff,	PRECIS_1981_1995_DELTA_BC_DEC.tiff,
# CanESM2_1981_1995_DELTA.BC_JAN.tiff,	CanESM2_1981_1995_DELTA.BC_FEB.tiff,	CanESM2_1981_1995_DELTA.BC_MAR.tiff,	CanESM2_1981_1995_DELTA.BC_APR.tiff,	CanESM2_1981_1995_DELTA.BC_MAY.tiff,	CanESM2_1981_1995_DELTA.BC_JUN.tiff,	CanESM2_1981_1995_DELTA.BC_JUL.tiff,	CanESM2_1981_1995_DELTA.BC_AUG.tiff,	CanESM2_1981_1995_DELTA.BC_SEP.tiff,	CanESM2_1981_1995_DELTA.BC_OCT.tiff,	CanESM2_1981_1995_DELTA.BC_NOV.tiff,	CanESM2_1981_1995_DELTA.BC_DEC.tiff,
# MIROC_1981_1995_DELTA_BC_JAN.tiff,	MIROC_1981_1995_DELTA_BC_FEB.tiff,	MIROC_1981_1995_DELTA_BC_MAR.tiff,	MIROC_1981_1995_DELTA_BC_APR.tiff,	MIROC_1981_1995_DELTA_BC_MAY.tiff,	MIROC_1981_1995_DELTA_BC_JUN.tiff,	MIROC_1981_1995_DELTA_BC_JUL.tiff,	MIROC_1981_1995_DELTA_BC_AUG.tiff,	MIROC_1981_1995_DELTA_BC_SEP.tiff,	MIROC_1981_1995_DELTA_BC_OCT.tiff,	MIROC_1981_1995_DELTA_BC_NOV.tiff,	MIROC_1981_1995_DELTA_BC_DEC.tiff,
# EARTH_1981_1995_DELTA_BC_JAN.tiff,	EARTH_1981_1995_DELTA_BC_FEB.tiff,	EARTH_1981_1995_DELTA_BC_MAR.tiff,	EARTH_1981_1995_DELTA_BC_APR.tiff,	EARTH_1981_1995_DELTA_BC_MAY.tiff,	EARTH_1981_1995_DELTA_BC_JUN.tiff,	EARTH_1981_1995_DELTA_BC_JUL.tiff,	EARTH_1981_1995_DELTA_BC_AUG.tiff,	EARTH_1981_1995_DELTA_BC_SEP.tiff,	EARTH_1981_1995_DELTA_BC_OCT.tiff,	EARTH_1981_1995_DELTA_BC_NOV.tiff,	EARTH_1981_1995_DELTA_BC_DEC.tiff,
# HadGEM2_1981_1995_DELTA_BC_JAN.tiff,	HadGEM2_1981_1995_DELTA_BC_FEB.tiff,	HadGEM2_1981_1995_DELTA_BC_MAR.tiff,	HadGEM2_1981_1995_DELTA_BC_APR.tiff,	HadGEM2_1981_1995_DELTA_BC_MAY.tiff,	HadGEM2_1981_1995_DELTA_BC_JUN.tiff,	HadGEM2_1981_1995_DELTA_BC_JUL.tiff,	HadGEM2_1981_1995_DELTA_BC_AUG.tiff,	HadGEM2_1981_1995_DELTA_BC_SEP.tiff,	HadGEM2_1981_1995_DELTA_BC_OCT.tiff,	HadGEM2_1981_1995_DELTA_BC_NOV.tiff,	HadGEM2_1981_1995_DELTA_BC_DEC.tiff,
# ENSAMBLE_1981_1995_DELTA_BC_JAN.tiff,	ENSAMBLE_1981_1995_DELTA_BC_FEB.tiff,	ENSAMBLE_1981_1995_DELTA_BC_MAR.tiff,	ENSAMBLE_1981_1995_DELTA_BC_APR.tiff,	ENSAMBLE_1981_1995_DELTA_BC_MAY.tiff,	ENSAMBLE_1981_1995_DELTA_BC_JUN.tiff,	ENSAMBLE_1981_1995_DELTA_BC_JUL.tiff,	ENSAMBLE_1981_1995_DELTA_BC_AUG.tiff,	ENSAMBLE_1981_1995_DELTA_BC_SEP.tiff,	ENSAMBLE_1981_1995_DELTA_BC_OCT.tiff,	ENSAMBLE_1981_1995_DELTA_BC_NOV.tiff,	ENSAMBLE_1981_1995_DELTA_BC_DEC.tiff,
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Workspace is cleared
#rm(list = ls())

# CRAN libraries are loaded
require(raster)
require(automap)
require(car)
require(chron)
require(compare)
require(DescTools)
require(doParallel)
require(downscaleR)
require(dplyr)
require(forecast)
require(ggmap)
require(ggplot2)
require(gstat)
require(hyfo)
require(intamap)
require(KScorrect)
require(lattice)
require(loadeR)
require(loadeR.2nc)
require(loadeR.java)
require(maps)
require(maptools)
require(MASS)
require(ncdf4)
require(ncdf4.helpers)
require(parallel)
require(pastecs)
require(PCICt)
require(plyr)
require(RColorBrewer)
require(rcompanion)
require(readxl)
require(reshape)
require(reshape2)
require(rgdal)
require(rgeos)
require(scales)
require(snow)
require(sp)
require(spatialEco)
require(tibble)
require(transformeR)

# Working directory is defined
setwd("/mnt/BACKUP/R_ITC/XLSX")  # UBUNTU-LINUX
setwd("C:/DATOS/R_ITC/XLSX")     # MS-WINDOWS

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: PRECIS georeference is loaded
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# The blank map is transformed into SpatialGridDataFrame
raster.precis <- read.asciigrid("mod_region.asc", as.image = FALSE)

# The map is transformed into RasterLayer
raster.precis <- raster(raster.precis)

# CRTM05 projection for CR is created
CRTM05 <- CRS("+proj=tmerc +lat_0=0 +lon_0=-84 +k=0.9999 +x_0=500000 +y_0=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs")

# CRTM05 projection is assigned to rasterbrick object
crs(raster.precis) <- CRTM05

# Object attributes are requested
# It can be seen that spa.res is different !!!!
image(raster.precis, col = topo.colors(64))

# A simple plot is requested
plot(raster.precis)

# A summery is requested
summary(raster.precis)

# A dummy raster is created
raster.precis.dummy <- raster.precis

# Dummy NA values are introduced
values(raster.precis.dummy) <- NA

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Execution of Bias Correction by loadeR/downscaleR 
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

V.select.month <- c("JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC")

for(i in 1:12) { # Main loop is initiated
  
  # Month of the year MUST be manually selected
  #i = ((12))
  
  # loadNcdf {hyfo} is used to load generic R NetCDF objects for the period 1951-1980
  #--------------------------------------------------------------------------------------------------------------------------------------
  St_ncin_hist_1951_1980_IMN <- loadNcdf(filePath="hist_1951_1980_IMN.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  St_ncin_hist_1951_1980_GPCC <- loadNcdf(filePath="hist_1951_1980_GPCC.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  St_ncin_hist_1951_1980_PRECIS <- loadNcdf(filePath="hist_1951_1980_PRECIS.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  St_ncin_hist_1951_1980_CanESM2 <- loadNcdf(filePath="Raw_crtm05_CanESM2_1951_1980.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  St_ncin_hist_1951_1980_MIROC <- loadNcdf(filePath="Raw_crtm05_MIROC5_1951_1980.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  St_ncin_hist_1951_1980_EARTH <- loadNcdf(filePath="Raw_crtm05_EARTH_1951_1980.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  St_ncin_hist_1951_1980_HadGEM2 <- loadNcdf(filePath="Raw_crtm05_HadGEM2_1951_1980.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  St_ncin_hist_1951_1980_MPI <- loadNcdf(filePath="Raw_crtm05_MPI_1951_1980.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  St_ncin_hist_1951_1980_ENSAMBLE <- loadNcdf(filePath="Raw_crtm05_ENSEMBLE_1951_1980.nc", varname="Precipitation", year = 1951:1980, month = i) #c(1:12)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # loadNcdf {hyfo} is used to load generic R NetCDF objects for the period 1981-1995
  #--------------------------------------------------------------------------------------------------------------------------------------
  St_ncin_hist_1981_1995_IMN <- loadNcdf(filePath="hist_1981_1995_IMN.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  St_ncin_hist_1981_1995_GPCC <- loadNcdf(filePath="hist_1981_1995_GPCC.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  St_ncin_hist_1981_1995_PRECIS <- loadNcdf(filePath="hist_1981_1995_PRECIS.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  St_ncin_hist_1981_1995_CanESM2 <- loadNcdf(filePath="Raw_crtm05_CanESM2_1981_1995.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  St_ncin_hist_1981_1995_MIROC <- loadNcdf(filePath="Raw_crtm05_MIROC5_1981_1995.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  St_ncin_hist_1981_1995_EARTH <- loadNcdf(filePath="Raw_crtm05_EARTH_1981_1995.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  St_ncin_hist_1981_1995_HadGEM2 <- loadNcdf(filePath="Raw_crtm05_HadGEM2_1981_1995.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  St_ncin_hist_1981_1995_MPI <- loadNcdf(filePath="Raw_crtm05_MPI_1981_1995.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  St_ncin_hist_1981_1995_ENSAMBLE <- loadNcdf(filePath="Raw_crtm05_ENSEMBLE_1981_1995.nc", varname="Precipitation", year = 1981:1995, month = i) #c(1:12)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # Corresponding WGS84 XY coordinates are converted from CRTM05 via ILWIS-GIS
  xxd <- c(-85.95864235, -85.72982083, -85.5009705,-85.27209516,-85.04319862,-84.81428469,-84.58535719,-84.35641994,-84.12747674,-83.89853141,-83.66958778,-83.44064966,-83.21172086,-82.9828052,-82.75390649,-82.52502855)
  yyd <- c(11.21234599,10.98639046,10.76043146,10.53446907,10.30850336,10.08253438,9.856562224,9.630586943,9.404608612,9.178627299,8.952643074,8.726656006,8.500666165,8.274673621,8.048678443)
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # {hyfo} Lists XY CRTM05 coordinates are replaced by WGS84 for 1951_1980 period
  #--------------------------------------------------------------------------------------------------------------------------------------
  St_ncin_hist_1951_1980_IMN$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_IMN$xyCoords$y <- (yyd)
  
  St_ncin_hist_1951_1980_GPCC$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_GPCC$xyCoords$y <- (yyd)
  
  St_ncin_hist_1951_1980_PRECIS$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_PRECIS$xyCoords$y <- (yyd)
  
  St_ncin_hist_1951_1980_CanESM2$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_CanESM2$xyCoords$y <- (yyd)
  
  St_ncin_hist_1951_1980_MIROC$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_MIROC$xyCoords$y <- (yyd)
  
  St_ncin_hist_1951_1980_EARTH$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_EARTH$xyCoords$y <- (yyd)
  
  St_ncin_hist_1951_1980_HadGEM2$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_HadGEM2$xyCoords$y <- (yyd)
  
  St_ncin_hist_1951_1980_MPI$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_MPI$xyCoords$y <- (yyd)
  
  St_ncin_hist_1951_1980_ENSAMBLE$xyCoords$x <- (xxd)
  St_ncin_hist_1951_1980_ENSAMBLE$xyCoords$y <- (yyd)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # {hyfo} Lists XY CRTM05 coordinates are replaced by WGS84 for 1981_1995 period
  #--------------------------------------------------------------------------------------------------------------------------------------
  St_ncin_hist_1981_1995_IMN$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_IMN$xyCoords$y <- (yyd)
  
  St_ncin_hist_1981_1995_GPCC$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_GPCC$xyCoords$y <- (yyd)
  
  St_ncin_hist_1981_1995_PRECIS$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_PRECIS$xyCoords$y <- (yyd)
  
  St_ncin_hist_1981_1995_CanESM2$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_CanESM2$xyCoords$y <- (yyd)
  
  St_ncin_hist_1981_1995_MIROC$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_MIROC$xyCoords$y <- (yyd)
  
  St_ncin_hist_1981_1995_EARTH$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_EARTH$xyCoords$y <- (yyd)
  
  St_ncin_hist_1981_1995_HadGEM2$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_HadGEM2$xyCoords$y <- (yyd)
  
  St_ncin_hist_1981_1995_MPI$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_MPI$xyCoords$y <- (yyd)
  
  St_ncin_hist_1981_1995_ENSAMBLE$xyCoords$x <- (xxd)
  St_ncin_hist_1981_1995_ENSAMBLE$xyCoords$y <- (yyd)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # {hyfo} Lists lat/long name attribues are re-printed within list-objects for 1951_1980 period
  #--------------------------------------------------------------------------------------------------------------------------------------
  names(St_ncin_hist_1951_1980_IMN$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_IMN$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1951_1980_GPCC$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_GPCC$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1951_1980_PRECIS$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_PRECIS$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1951_1980_CanESM2$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_CanESM2$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1951_1980_MIROC$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_MIROC$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1951_1980_EARTH$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_EARTH$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1951_1980_HadGEM2$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_HadGEM2$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1951_1980_MPI$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_MPI$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1951_1980_ENSAMBLE$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1951_1980_ENSAMBLE$xyCoords$y) <- c("lat")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # {hyfo} Lists lat/long name attribues are re-printed within list-objects for 1981_1995 period
  #--------------------------------------------------------------------------------------------------------------------------------------
  names(St_ncin_hist_1981_1995_IMN$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_IMN$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1981_1995_GPCC$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_GPCC$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1981_1995_PRECIS$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_PRECIS$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1981_1995_CanESM2$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_CanESM2$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1981_1995_MIROC$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_MIROC$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1981_1995_EARTH$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_EARTH$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1981_1995_HadGEM2$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_HadGEM2$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1981_1995_MPI$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_MPI$xyCoords$y) <- c("lat")
  
  names(St_ncin_hist_1981_1995_ENSAMBLE$xyCoords$x) <- c("lon")
  names(St_ncin_hist_1981_1995_ENSAMBLE$xyCoords$y) <- c("lat")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # writeNcdf {hyfo} is used to externally saved NetCDF version 4.0 for 1951_1980 period
  #--------------------------------------------------------------------------------------------------------------------------------------
  writeNcdf(gridData = St_ncin_hist_1951_1980_IMN, filePath = "St_ncin_hist_1951_1980_IMN.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1951_1980_GPCC, filePath = "St_ncin_hist_1951_1980_GPCC.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1951_1980_PRECIS, filePath = "St_ncin_hist_1951_1980_PRECIS.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1951_1980_CanESM2, filePath = "St_ncin_hist_1951_1980_CanESM2.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1951_1980_MIROC, filePath = "St_ncin_hist_1951_1980_MIROC.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1951_1980_EARTH, filePath = "St_ncin_hist_1951_1980_EARTH.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1951_1980_HadGEM2, filePath = "St_ncin_hist_1951_1980_HadGEM2.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1951_1980_MPI, filePath = "St_ncin_hist_1951_1980_MPI.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1951_1980_ENSAMBLE, filePath = "St_ncin_hist_1951_1980_ENSAMBLE.nc", version = 4)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # writeNcdf {hyfo} is used to externally saved NetCDF version 4.0 for 1981_1995 period
  #--------------------------------------------------------------------------------------------------------------------------------------
  writeNcdf(gridData = St_ncin_hist_1981_1995_IMN, filePath = "St_ncin_hist_1981_1995_IMN.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1981_1995_GPCC, filePath = "St_ncin_hist_1981_1995_GPCC.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1981_1995_PRECIS, filePath = "St_ncin_hist_1981_1995_PRECIS.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1981_1995_CanESM2, filePath = "St_ncin_hist_1981_1995_CanESM2.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1981_1995_MIROC, filePath = "St_ncin_hist_1981_1995_MIROC.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1981_1995_EARTH, filePath = "St_ncin_hist_1981_1995_EARTH.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1981_1995_HadGEM2, filePath = "St_ncin_hist_1981_1995_HadGEM2.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1981_1995_MPI, filePath = "St_ncin_hist_1981_1995_MPI.nc", version = 4)
  writeNcdf(gridData = St_ncin_hist_1981_1995_ENSAMBLE, filePath = "St_ncin_hist_1981_1995_ENSAMBLE.nc", version = 4)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # dataInventory {loadeR} is used to check the NetCDF integrity within Santander-Group 
  dataInventory("St_ncin_hist_1951_1980_PRECIS.nc")
  dataInventory("St_ncin_hist_1951_1980_CanESM2.nc")
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # loadGridData {loadeR} is used to create Santander-Group NetCDF binary lists for 1951_1980 period
  #--------------------------------------------------------------------------------------------------------------------------------------
  ncin_hist_1951_1980_IMN <- loadGridData("St_ncin_hist_1951_1980_IMN.nc", var = "Precipitation")
  ncin_hist_1951_1980_GPCC <- loadGridData("St_ncin_hist_1951_1980_GPCC.nc", var = "Precipitation")
  ncin_hist_1951_1980_PRECIS <- loadGridData("St_ncin_hist_1951_1980_PRECIS.nc", var = "Precipitation")
  ncin_hist_1951_1980_CanESM2 <- loadGridData("St_ncin_hist_1951_1980_CanESM2.nc", var = "Precipitation")
  ncin_hist_1951_1980_MIROC <- loadGridData("St_ncin_hist_1951_1980_MIROC.nc", var = "Precipitation")
  ncin_hist_1951_1980_EARTH <- loadGridData("St_ncin_hist_1951_1980_EARTH.nc", var = "Precipitation")
  ncin_hist_1951_1980_HadGEM2 <- loadGridData("St_ncin_hist_1951_1980_HadGEM2.nc", var = "Precipitation")
  ncin_hist_1951_1980_MPI <- loadGridData("St_ncin_hist_1951_1980_MPI.nc", var = "Precipitation")
  ncin_hist_1951_1980_ENSAMBLE <- loadGridData("St_ncin_hist_1951_1980_ENSAMBLE.nc", var = "Precipitation")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # loadGridData {loadeR} is used to create Santander-Group NetCDF binary lists for 1981_1995 period
  #--------------------------------------------------------------------------------------------------------------------------------------
  ncin_hist_1981_1995_IMN <- loadGridData("St_ncin_hist_1981_1995_IMN.nc", var = "Precipitation")
  ncin_hist_1981_1995_GPCC <- loadGridData("St_ncin_hist_1981_1995_GPCC.nc", var = "Precipitation")
  ncin_hist_1981_1995_PRECIS <- loadGridData("St_ncin_hist_1981_1995_PRECIS.nc", var = "Precipitation")
  ncin_hist_1981_1995_CanESM2 <- loadGridData("St_ncin_hist_1981_1995_CanESM2.nc", var = "Precipitation")
  ncin_hist_1981_1995_MIROC <- loadGridData("St_ncin_hist_1981_1995_MIROC.nc", var = "Precipitation")
  ncin_hist_1981_1995_EARTH <- loadGridData("St_ncin_hist_1981_1995_EARTH.nc", var = "Precipitation")
  ncin_hist_1981_1995_HadGEM2 <- loadGridData("St_ncin_hist_1981_1995_HadGEM2.nc", var = "Precipitation")
  ncin_hist_1981_1995_MPI <- loadGridData("St_ncin_hist_1981_1995_MPI.nc", var = "Precipitation")
  ncin_hist_1981_1995_ENSAMBLE <- loadGridData("St_ncin_hist_1981_1995_ENSAMBLE.nc", var = "Precipitation")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # Temporary NetCDF files created by grid2nc {loadeR.2nc} are removed
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  file.remove("Temp_GPCC_1981_1995.GPQM.BC.nc") 
  file.remove("Temp_PRECIS_1981_1995.GPQM.BC.nc") 
  file.remove("Temp_CanESM2_1981_1995.GPQM.BC.nc") 
  file.remove("Temp_MIROC_1981_1995.GPQM.BC.nc") 
  file.remove("Temp_EARTH_1981_1995.GPQM.BC.nc") 
  file.remove("Temp_HadGEM2_1981_1995.GPQM.BC.nc") 
  file.remove("Temp_MPI_1981_1995.GPQM.BC.nc")
  file.remove("Temp_ENSAMBLE_1981_1995.GPQM.BC.nc")
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  file.remove("Temp_GPCC_1981_1995.DELTA.BC.nc") 
  file.remove("Temp_PRECIS_1981_1995.DELTA.BC.nc") 
  file.remove("Temp_CanESM2_1981_1995.DELTA.BC.nc") 
  file.remove("Temp_MIROC_1981_1995.DELTA.BC.nc") 
  file.remove("Temp_EARTH_1981_1995.DELTA.BC.nc") 
  file.remove("Temp_HadGEM2_1981_1995.DELTA.BC.nc") 
  file.remove("Temp_MPI_1981_1995.DELTA.BC.nc")
  file.remove("Temp_ENSAMBLE_1981_1995.DELTA.BC.nc")
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  file.remove("Temp_GPCC_1981_1995.PTR.BC.nc") 
  file.remove("Temp_PRECIS_1981_1995.PTR.BC.nc") 
  file.remove("Temp_CanESM2_1981_1995.PTR.BC.nc") 
  file.remove("Temp_MIROC_1981_1995.PTR.BC.nc") 
  file.remove("Temp_EARTH_1981_1995.PTR.BC.nc") 
  file.remove("Temp_HadGEM2_1981_1995.PTR.BC.nc") 
  file.remove("Temp_MPI_1981_1995.PTR.BC.nc")
  file.remove("Temp_ENSAMBLE_1981_1995.PTR.BC.nc")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # biasCorrection {downscaleR} is used for bias correction according to Santander-Group
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  Temp_GPCC_1981_1995.GPQM.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_GPCC, newdata = ncin_hist_1981_1995_GPCC, method = "gpqm", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_PRECIS_1981_1995.GPQM.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_PRECIS, newdata = ncin_hist_1981_1995_PRECIS, method = "gpqm", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_CanESM2_1981_1995.GPQM.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_CanESM2, newdata = ncin_hist_1981_1995_CanESM2, method = "gpqm", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_MIROC_1981_1995.GPQM.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_MIROC, newdata = ncin_hist_1981_1995_MIROC, method = "gpqm", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_EARTH_1981_1995.GPQM.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_EARTH, newdata = ncin_hist_1981_1995_EARTH, method = "gpqm", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_HadGEM2_1981_1995.GPQM.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_HadGEM2, newdata = ncin_hist_1981_1995_HadGEM2, method = "gpqm", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_MPI_1981_1995.GPQM.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_MPI, newdata = ncin_hist_1981_1995_MPI, method = "gpqm", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_ENSAMBLE_1981_1995.GPQM.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_ENSAMBLE, newdata = ncin_hist_1981_1995_ENSAMBLE, method = "gpqm", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  Temp_GPCC_1981_1995.DELTA.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_GPCC, newdata = ncin_hist_1981_1995_GPCC, method = "delta", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_PRECIS_1981_1995.DELTA.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_PRECIS, newdata = ncin_hist_1981_1995_PRECIS, method = "delta", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_CanESM2_1981_1995.DELTA.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_CanESM2, newdata = ncin_hist_1981_1995_CanESM2, method = "delta", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_MIROC_1981_1995.DELTA.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_MIROC, newdata = ncin_hist_1981_1995_MIROC, method = "delta", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_EARTH_1981_1995.DELTA.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_EARTH, newdata = ncin_hist_1981_1995_EARTH, method = "delta", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_HadGEM2_1981_1995.DELTA.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_HadGEM2, newdata = ncin_hist_1981_1995_HadGEM2, method = "delta", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_MPI_1981_1995.DELTA.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_MPI, newdata = ncin_hist_1981_1995_MPI, method = "delta", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_ENSAMBLE_1981_1995.DELTA.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_ENSAMBLE, newdata = ncin_hist_1981_1995_ENSAMBLE, method = "delta", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  Temp_GPCC_1981_1995.PTR.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_GPCC, newdata = ncin_hist_1981_1995_GPCC, method = "ptr", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_PRECIS_1981_1995.PTR.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_PRECIS, newdata = ncin_hist_1981_1995_PRECIS, method = "ptr", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_CanESM2_1981_1995.PTR.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_CanESM2, newdata = ncin_hist_1981_1995_CanESM2, method = "ptr", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_MIROC_1981_1995.PTR.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_MIROC, newdata = ncin_hist_1981_1995_MIROC, method = "ptr", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_EARTH_1981_1995.PTR.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_EARTH, newdata = ncin_hist_1981_1995_EARTH, method = "ptr", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_HadGEM2_1981_1995.PTR.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_HadGEM2, newdata = ncin_hist_1981_1995_HadGEM2, method = "ptr", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_MPI_1981_1995.PTR.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_MPI, newdata = ncin_hist_1981_1995_MPI, method = "ptr", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  Temp_ENSAMBLE_1981_1995.PTR.BC <- biasCorrection(y = ncin_hist_1951_1980_IMN, x = ncin_hist_1951_1980_ENSAMBLE, newdata = ncin_hist_1981_1995_ENSAMBLE, method = "ptr", wet.threshold = 0, precipitation = TRUE, max.ncores = 16, ncores = 16)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # Corresponding CRTM05 XY coordinates are converted from WGS84 via ILWIS-GIS
  xxOUT <- c(286080, 311080, 336080, 361080, 386080, 411080, 436080, 461080, 486080, 511080, 536080, 561080, 586080, 611080, 636080, 661080)
  yyOUT <- c(1240234, 1215234, 1190234, 1165234, 1140234, 1115234, 1090234, 1065234, 1040234, 1015234,  990234,  965234,  940234,  915234,  890234)
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # Santander-Group list coordinates Lists XY WGS84 coordinates are replaced by CRTM05
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  Temp_GPCC_1981_1995.GPQM.BC$xyCoords$x <- xxOUT
  Temp_PRECIS_1981_1995.GPQM.BC$xyCoords$x <- xxOUT
  Temp_CanESM2_1981_1995.GPQM.BC$xyCoords$x <- xxOUT
  Temp_MIROC_1981_1995.GPQM.BC$xyCoords$x <- xxOUT
  Temp_EARTH_1981_1995.GPQM.BC$xyCoords$x <- xxOUT
  Temp_HadGEM2_1981_1995.GPQM.BC$xyCoords$x <- xxOUT
  Temp_MPI_1981_1995.GPQM.BC$xyCoords$x <- xxOUT
  Temp_ENSAMBLE_1981_1995.GPQM.BC$xyCoords$x <- xxOUT
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  Temp_GPCC_1981_1995.DELTA.BC$xyCoords$x <- xxOUT
  Temp_PRECIS_1981_1995.DELTA.BC$xyCoords$x <- xxOUT
  Temp_CanESM2_1981_1995.DELTA.BC$xyCoords$x <- xxOUT
  Temp_MIROC_1981_1995.DELTA.BC$xyCoords$x <- xxOUT
  Temp_EARTH_1981_1995.DELTA.BC$xyCoords$x <- xxOUT
  Temp_HadGEM2_1981_1995.DELTA.BC$xyCoords$x <- xxOUT
  Temp_MPI_1981_1995.DELTA.BC$xyCoords$x <- xxOUT
  Temp_ENSAMBLE_1981_1995.DELTA.BC$xyCoords$x <- xxOUT
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  Temp_GPCC_1981_1995.PTR.BC$xyCoords$x <- xxOUT
  Temp_PRECIS_1981_1995.PTR.BC$xyCoords$x <- xxOUT
  Temp_CanESM2_1981_1995.PTR.BC$xyCoords$x <- xxOUT
  Temp_MIROC_1981_1995.PTR.BC$xyCoords$x <- xxOUT
  Temp_EARTH_1981_1995.PTR.BC$xyCoords$x <- xxOUT
  Temp_HadGEM2_1981_1995.PTR.BC$xyCoords$x <- xxOUT
  Temp_MPI_1981_1995.PTR.BC$xyCoords$x <- xxOUT
  Temp_ENSAMBLE_1981_1995.PTR.BC$xyCoords$x <- xxOUT
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # Santander-Group list coordinates Lists XY WGS84 coordinates are replaced by CRTM05
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  Temp_GPCC_1981_1995.GPQM.BC$xyCoords$y <- yyOUT
  Temp_PRECIS_1981_1995.GPQM.BC$xyCoords$y <- yyOUT
  Temp_CanESM2_1981_1995.GPQM.BC$xyCoords$y <- yyOUT
  Temp_MIROC_1981_1995.GPQM.BC$xyCoords$y <- yyOUT
  Temp_EARTH_1981_1995.GPQM.BC$xyCoords$y <- yyOUT
  Temp_HadGEM2_1981_1995.GPQM.BC$xyCoords$y <- yyOUT
  Temp_MPI_1981_1995.GPQM.BC$xyCoords$y <- yyOUT
  Temp_ENSAMBLE_1981_1995.GPQM.BC$xyCoords$y <- yyOUT
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  Temp_GPCC_1981_1995.DELTA.BC$xyCoords$y <- yyOUT
  Temp_PRECIS_1981_1995.DELTA.BC$xyCoords$y <- yyOUT
  Temp_CanESM2_1981_1995.DELTA.BC$xyCoords$y <- yyOUT
  Temp_MIROC_1981_1995.DELTA.BC$xyCoords$y <- yyOUT
  Temp_EARTH_1981_1995.DELTA.BC$xyCoords$y <- yyOUT
  Temp_HadGEM2_1981_1995.DELTA.BC$xyCoords$y <- yyOUT
  Temp_MPI_1981_1995.DELTA.BC$xyCoords$y <- yyOUT
  Temp_ENSAMBLE_1981_1995.DELTA.BC$xyCoords$y <- yyOUT
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  Temp_GPCC_1981_1995.PTR.BC$xyCoords$y <- yyOUT
  Temp_PRECIS_1981_1995.PTR.BC$xyCoords$y <- yyOUT
  Temp_CanESM2_1981_1995.PTR.BC$xyCoords$y <- yyOUT
  Temp_MIROC_1981_1995.PTR.BC$xyCoords$y <- yyOUT
  Temp_EARTH_1981_1995.PTR.BC$xyCoords$y <- yyOUT
  Temp_HadGEM2_1981_1995.PTR.BC$xyCoords$y <- yyOUT
  Temp_MPI_1981_1995.PTR.BC$xyCoords$y <- yyOUT
  Temp_ENSAMBLE_1981_1995.PTR.BC$xyCoords$y <- yyOUT
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # grid2nc {loadeR.2nc} is used to externally export temporary NetCDF files
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  grid2nc(Temp_GPCC_1981_1995.GPQM.BC, NetCDFOutFile = "Temp_GPCC_1981_1995.GPQM.nc")
  grid2nc(Temp_PRECIS_1981_1995.GPQM.BC, NetCDFOutFile = "Temp_PRECIS_1981_1995.GPQM.nc")
  grid2nc(Temp_CanESM2_1981_1995.GPQM.BC, NetCDFOutFile = "Temp_CanESM2_1981_1995.GPQM.nc")
  grid2nc(Temp_MIROC_1981_1995.GPQM.BC, NetCDFOutFile = "Temp_MIROC_1981_1995.GPQM.nc")
  grid2nc(Temp_EARTH_1981_1995.GPQM.BC, NetCDFOutFile = "Temp_EARTH_1981_1995.GPQM.nc")
  grid2nc(Temp_HadGEM2_1981_1995.GPQM.BC, NetCDFOutFile = "Temp_HadGEM2_1981_1995.GPQM.nc")
  grid2nc(Temp_MPI_1981_1995.GPQM.BC, NetCDFOutFile = "Temp_MPI_1981_1995.GPQM.nc")
  grid2nc(Temp_ENSAMBLE_1981_1995.GPQM.BC, NetCDFOutFile = "Temp_ENSAMBLE_1981_1995.GPQM.nc")
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  grid2nc(Temp_GPCC_1981_1995.DELTA.BC, NetCDFOutFile = "Temp_GPCC_1981_1995.DELTA.nc")
  grid2nc(Temp_PRECIS_1981_1995.DELTA.BC, NetCDFOutFile = "Temp_PRECIS_1981_1995.DELTA.nc")
  grid2nc(Temp_CanESM2_1981_1995.DELTA.BC, NetCDFOutFile = "Temp_CanESM2_1981_1995.DELTA.nc")
  grid2nc(Temp_MIROC_1981_1995.DELTA.BC, NetCDFOutFile = "Temp_MIROC_1981_1995.DELTA.nc")
  grid2nc(Temp_EARTH_1981_1995.DELTA.BC, NetCDFOutFile = "Temp_EARTH_1981_1995.DELTA.nc")
  grid2nc(Temp_HadGEM2_1981_1995.DELTA.BC, NetCDFOutFile = "Temp_HadGEM2_1981_1995.DELTA.nc")
  grid2nc(Temp_MPI_1981_1995.DELTA.BC, NetCDFOutFile = "Temp_MPI_1981_1995.DELTA.nc")
  grid2nc(Temp_ENSAMBLE_1981_1995.DELTA.BC, NetCDFOutFile = "Temp_ENSAMBLE_1981_1995.DELTA.nc")
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  grid2nc(Temp_GPCC_1981_1995.PTR.BC, NetCDFOutFile = "Temp_GPCC_1981_1995.PTR.nc")
  grid2nc(Temp_PRECIS_1981_1995.PTR.BC, NetCDFOutFile = "Temp_PRECIS_1981_1995.PTR.nc")
  grid2nc(Temp_CanESM2_1981_1995.PTR.BC, NetCDFOutFile = "Temp_CanESM2_1981_1995.PTR.nc")
  grid2nc(Temp_MIROC_1981_1995.PTR.BC, NetCDFOutFile = "Temp_MIROC_1981_1995.PTR.nc")
  grid2nc(Temp_EARTH_1981_1995.PTR.BC, NetCDFOutFile = "Temp_EARTH_1981_1995.PTR.nc")
  grid2nc(Temp_HadGEM2_1981_1995.PTR.BC, NetCDFOutFile = "Temp_HadGEM2_1981_1995.PTR.nc")
  grid2nc(Temp_MPI_1981_1995.PTR.BC, NetCDFOutFile = "Temp_MPI_1981_1995.PTR.nc")
  grid2nc(Temp_ENSAMBLE_1981_1995.PTR.BC, NetCDFOutFile = "Temp_ENSAMBLE_1981_1995.PTR.nc")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # loadNcdf {hyfo} is used to load grid2nc {loadeR.2nc} NetCDF objects
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  HTemp_GPCC_1981_1995.GPQM <- loadNcdf(filePath= "Temp_GPCC_1981_1995.GPQM.nc", varname = "Precipitation")
  HTemp_PRECIS_1981_1995.GPQM <- loadNcdf(filePath= "Temp_PRECIS_1981_1995.GPQM.nc", varname = "Precipitation")
  HTemp_CanESM2_1981_1995.GPQM <- loadNcdf(filePath= "Temp_CanESM2_1981_1995.GPQM.nc", varname = "Precipitation")
  HTemp_MIROC_1981_1995.GPQM <- loadNcdf(filePath= "Temp_MIROC_1981_1995.GPQM.nc", varname = "Precipitation")
  HTemp_EARTH_1981_1995.GPQM <- loadNcdf(filePath= "Temp_EARTH_1981_1995.GPQM.nc", varname = "Precipitation")
  HTemp_HadGEM2_1981_1995.GPQM <- loadNcdf(filePath= "Temp_HadGEM2_1981_1995.GPQM.nc", varname = "Precipitation")
  HTemp_MPI_1981_1995.GPQM <- loadNcdf(filePath= "Temp_MPI_1981_1995.GPQM.nc", varname = "Precipitation")
  HTemp_ENSAMBLE_1981_1995.GPQM <- loadNcdf(filePath= "Temp_ENSAMBLE_1981_1995.GPQM.nc", varname = "Precipitation")
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  HTemp_GPCC_1981_1995.DELTA <- loadNcdf(filePath= "Temp_GPCC_1981_1995.DELTA.nc", varname = "Precipitation")
  HTemp_PRECIS_1981_1995.DELTA <- loadNcdf(filePath= "Temp_PRECIS_1981_1995.DELTA.nc", varname = "Precipitation")
  HTemp_CanESM2_1981_1995.DELTA <- loadNcdf(filePath= "Temp_CanESM2_1981_1995.DELTA.nc", varname = "Precipitation")
  HTemp_MIROC_1981_1995.DELTA <- loadNcdf(filePath= "Temp_MIROC_1981_1995.DELTA.nc", varname = "Precipitation")
  HTemp_EARTH_1981_1995.DELTA <- loadNcdf(filePath= "Temp_EARTH_1981_1995.DELTA.nc", varname = "Precipitation")
  HTemp_HadGEM2_1981_1995.DELTA <- loadNcdf(filePath= "Temp_HadGEM2_1981_1995.DELTA.nc", varname = "Precipitation")
  HTemp_MPI_1981_1995.DELTA <- loadNcdf(filePath= "Temp_MPI_1981_1995.DELTA.nc", varname = "Precipitation")
  HTemp_ENSAMBLE_1981_1995.DELTA <- loadNcdf(filePath= "Temp_ENSAMBLE_1981_1995.DELTA.nc", varname = "Precipitation")
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  HTemp_GPCC_1981_1995.PTR <- loadNcdf(filePath= "Temp_GPCC_1981_1995.PTR.nc", varname = "Precipitation")
  HTemp_PRECIS_1981_1995.PTR <- loadNcdf(filePath= "Temp_PRECIS_1981_1995.PTR.nc", varname = "Precipitation")
  HTemp_CanESM2_1981_1995.PTR <- loadNcdf(filePath= "Temp_CanESM2_1981_1995.PTR.nc", varname = "Precipitation")
  HTemp_MIROC_1981_1995.PTR <- loadNcdf(filePath= "Temp_MIROC_1981_1995.PTR.nc", varname = "Precipitation")
  HTemp_EARTH_1981_1995.PTR <- loadNcdf(filePath= "Temp_EARTH_1981_1995.PTR.nc", varname = "Precipitation")
  HTemp_HadGEM2_1981_1995.PTR <- loadNcdf(filePath= "Temp_HadGEM2_1981_1995.PTR.nc", varname = "Precipitation")
  HTemp_MPI_1981_1995.PTR <- loadNcdf(filePath= "Temp_MPI_1981_1995.PTR.nc", varname = "Precipitation")
  HTemp_ENSAMBLE_1981_1995.PTR <- loadNcdf(filePath= "Temp_ENSAMBLE_1981_1995.PTR.nc", varname = "Precipitation")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # writeNcdf {hyfo} is used to externally saved NetCDF version 4.0
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  writeNcdf(gridData = HTemp_GPCC_1981_1995.GPQM, filePath = "HTemp_GPCC_1981_1995.GPQM.nc", version = 4)
  writeNcdf(gridData = HTemp_PRECIS_1981_1995.GPQM, filePath = "HTemp_PRECIS_1981_1995.GPQM.nc", version = 4)
  writeNcdf(gridData = HTemp_CanESM2_1981_1995.GPQM, filePath = "HTemp_CanESM2_1981_1995.GPQM.nc", version = 4)
  writeNcdf(gridData = HTemp_MIROC_1981_1995.GPQM, filePath = "HTemp_MIROC_1981_1995.GPQM.nc", version = 4)
  writeNcdf(gridData = HTemp_EARTH_1981_1995.GPQM, filePath = "HTemp_EARTH_1981_1995.GPQM.nc", version = 4)
  writeNcdf(gridData = HTemp_HadGEM2_1981_1995.GPQM, filePath = "HTemp_HadGEM2_1981_1995.GPQM.nc", version = 4)
  writeNcdf(gridData = HTemp_MPI_1981_1995.GPQM, filePath = "HTemp_MPI_1981_1995.GPQM.nc", version = 4)
  writeNcdf(gridData = HTemp_ENSAMBLE_1981_1995.GPQM, filePath = "HTemp_ENSAMBLE_1981_1995.GPQM.nc", version = 4)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  writeNcdf(gridData = HTemp_GPCC_1981_1995.DELTA, filePath = "HTemp_GPCC_1981_1995.DELTA.nc", version = 4)
  writeNcdf(gridData = HTemp_PRECIS_1981_1995.DELTA, filePath = "HTemp_PRECIS_1981_1995.DELTA.nc", version = 4)
  writeNcdf(gridData = HTemp_CanESM2_1981_1995.DELTA, filePath = "HTemp_CanESM2_1981_1995.DELTA.nc", version = 4)
  writeNcdf(gridData = HTemp_MIROC_1981_1995.DELTA, filePath = "HTemp_MIROC_1981_1995.DELTA.nc", version = 4)
  writeNcdf(gridData = HTemp_EARTH_1981_1995.DELTA, filePath = "HTemp_EARTH_1981_1995.DELTA.nc", version = 4)
  writeNcdf(gridData = HTemp_HadGEM2_1981_1995.DELTA, filePath = "HTemp_HadGEM2_1981_1995.DELTA.nc", version = 4)
  writeNcdf(gridData = HTemp_MPI_1981_1995.DELTA, filePath = "HTemp_MPI_1981_1995.DELTA.nc", version = 4)
  writeNcdf(gridData = HTemp_ENSAMBLE_1981_1995.DELTA, filePath = "HTemp_ENSAMBLE_1981_1995.DELTA.nc", version = 4)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  writeNcdf(gridData = HTemp_GPCC_1981_1995.PTR, filePath = "HTemp_GPCC_1981_1995.PTR.nc", version = 4)
  writeNcdf(gridData = HTemp_PRECIS_1981_1995.PTR, filePath = "HTemp_PRECIS_1981_1995.PTR.nc", version = 4)
  writeNcdf(gridData = HTemp_CanESM2_1981_1995.PTR, filePath = "HTemp_CanESM2_1981_1995.PTR.nc", version = 4)
  writeNcdf(gridData = HTemp_MIROC_1981_1995.PTR, filePath = "HTemp_MIROC_1981_1995.PTR.nc", version = 4)
  writeNcdf(gridData = HTemp_EARTH_1981_1995.PTR, filePath = "HTemp_EARTH_1981_1995.PTR.nc", version = 4)
  writeNcdf(gridData = HTemp_HadGEM2_1981_1995.PTR, filePath = "HTemp_HadGEM2_1981_1995.PTR.nc", version = 4)
  writeNcdf(gridData = HTemp_MPI_1981_1995.PTR, filePath = "HTemp_MPI_1981_1995.PTR.nc", version = 4)
  writeNcdf(gridData = HTemp_ENSAMBLE_1981_1995.PTR, filePath = "HTemp_ENSAMBLE_1981_1995.PTR.nc", version = 4)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # stack {raster} is used to create a RasterStack object
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  HTemp_GPCC_1981_1995.GPQM.raster <- stack("HTemp_GPCC_1981_1995.GPQM.nc")
  HTemp_PRECIS_1981_1995.GPQM.raster <- stack("HTemp_PRECIS_1981_1995.GPQM.nc")
  HTemp_CanESM2_1981_1995.GPQM.raster <- stack("HTemp_CanESM2_1981_1995.GPQM.nc")
  HTemp_MIROC_1981_1995.GPQM.raster <- stack("HTemp_MIROC_1981_1995.GPQM.nc")
  HTemp_EARTH_1981_1995.GPQM.raster <- stack("HTemp_EARTH_1981_1995.GPQM.nc")
  HTemp_HadGEM2_1981_1995.GPQM.raster <- stack("HTemp_HadGEM2_1981_1995.GPQM.nc")
  HTemp_MPI_1981_1995.GPQM.raster <- stack("HTemp_MPI_1981_1995.GPQM.nc")
  HTemp_ENSAMBLE_1981_1995.GPQM.raster <- stack("HTemp_ENSAMBLE_1981_1995.GPQM.nc")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  HTemp_GPCC_1981_1995.DELTA.raster <- stack("HTemp_GPCC_1981_1995.DELTA.nc")
  HTemp_PRECIS_1981_1995.DELTA.raster <- stack("HTemp_PRECIS_1981_1995.DELTA.nc")
  HTemp_CanESM2_1981_1995.DELTA.raster <- stack("HTemp_CanESM2_1981_1995.DELTA.nc")
  HTemp_MIROC_1981_1995.DELTA.raster <- stack("HTemp_MIROC_1981_1995.DELTA.nc")
  HTemp_EARTH_1981_1995.DELTA.raster <- stack("HTemp_EARTH_1981_1995.DELTA.nc")
  HTemp_HadGEM2_1981_1995.DELTA.raster <- stack("HTemp_HadGEM2_1981_1995.DELTA.nc")
  HTemp_MPI_1981_1995.DELTA.raster <- stack("HTemp_MPI_1981_1995.DELTA.nc")
  HTemp_ENSAMBLE_1981_1995.DELTA.raster <- stack("HTemp_ENSAMBLE_1981_1995.DELTA.nc")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  HTemp_GPCC_1981_1995.PTR.raster <- stack("HTemp_GPCC_1981_1995.PTR.nc")
  HTemp_PRECIS_1981_1995.PTR.raster <- stack("HTemp_PRECIS_1981_1995.PTR.nc")
  HTemp_CanESM2_1981_1995.PTR.raster <- stack("HTemp_CanESM2_1981_1995.PTR.nc")
  HTemp_MIROC_1981_1995.PTR.raster <- stack("HTemp_MIROC_1981_1995.PTR.nc")
  HTemp_EARTH_1981_1995.PTR.raster <- stack("HTemp_EARTH_1981_1995.PTR.nc")
  HTemp_HadGEM2_1981_1995.PTR.raster <- stack("HTemp_HadGEM2_1981_1995.PTR.nc")
  HTemp_MPI_1981_1995.PTR.raster <- stack("HTemp_MPI_1981_1995.PTR.nc")
  HTemp_ENSAMBLE_1981_1995.PTR.raster <- stack("HTemp_ENSAMBLE_1981_1995.PTR.nc")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # stack {flip} is used to flip a RasterStack object
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  HTemp_GPCC_1981_1995.GPQM.raster <- flip(HTemp_GPCC_1981_1995.GPQM.raster, direction ="y")
  HTemp_PRECIS_1981_1995.GPQM.raster <- flip(HTemp_PRECIS_1981_1995.GPQM.raster, direction ="y")
  HTemp_CanESM2_1981_1995.GPQM.raster <- flip(HTemp_CanESM2_1981_1995.GPQM.raster, direction ="y")
  HTemp_MIROC_1981_1995.GPQM.raster <- flip(HTemp_MIROC_1981_1995.GPQM.raster, direction ="y")
  HTemp_EARTH_1981_1995.GPQM.raster <- flip(HTemp_EARTH_1981_1995.GPQM.raster, direction ="y")
  HTemp_HadGEM2_1981_1995.GPQM.raster <- flip(HTemp_HadGEM2_1981_1995.GPQM.raster, direction ="y")
  HTemp_MPI_1981_1995.GPQM.raster <- flip(HTemp_MPI_1981_1995.GPQM.raster, direction ="y")
  HTemp_ENSAMBLE_1981_1995.GPQM.raster <- flip(HTemp_ENSAMBLE_1981_1995.GPQM.raster, direction ="y")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  HTemp_GPCC_1981_1995.DELTA.raster <- flip(HTemp_GPCC_1981_1995.DELTA.raster, direction ="y")
  HTemp_PRECIS_1981_1995.DELTA.raster <- flip(HTemp_PRECIS_1981_1995.DELTA.raster, direction ="y")
  HTemp_CanESM2_1981_1995.DELTA.raster <- flip(HTemp_CanESM2_1981_1995.DELTA.raster, direction ="y")
  HTemp_MIROC_1981_1995.DELTA.raster <- flip(HTemp_MIROC_1981_1995.DELTA.raster, direction ="y")
  HTemp_EARTH_1981_1995.DELTA.raster <- flip(HTemp_EARTH_1981_1995.DELTA.raster, direction ="y")
  HTemp_HadGEM2_1981_1995.DELTA.raster <- flip(HTemp_HadGEM2_1981_1995.DELTA.raster, direction ="y")
  HTemp_MPI_1981_1995.DELTA.raster <- flip(HTemp_MPI_1981_1995.DELTA.raster, direction ="y")
  HTemp_ENSAMBLE_1981_1995.DELTA.raster <- flip(HTemp_ENSAMBLE_1981_1995.DELTA.raster, direction ="y")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  HTemp_GPCC_1981_1995.PTR.raster <- flip(HTemp_GPCC_1981_1995.PTR.raster, direction ="y")
  HTemp_PRECIS_1981_1995.PTR.raster <- flip(HTemp_PRECIS_1981_1995.PTR.raster, direction ="y")
  HTemp_CanESM2_1981_1995.PTR.raster <- flip(HTemp_CanESM2_1981_1995.PTR.raster, direction ="y")
  HTemp_MIROC_1981_1995.PTR.raster <- flip(HTemp_MIROC_1981_1995.PTR.raster, direction ="y")
  HTemp_EARTH_1981_1995.PTR.raster <- flip(HTemp_EARTH_1981_1995.PTR.raster, direction ="y")
  HTemp_HadGEM2_1981_1995.PTR.raster <- flip(HTemp_HadGEM2_1981_1995.PTR.raster, direction ="y")
  HTemp_MPI_1981_1995.PTR.raster <- flip(HTemp_MPI_1981_1995.PTR.raster, direction ="y")
  HTemp_ENSAMBLE_1981_1995.PTR.raster <- flip(HTemp_ENSAMBLE_1981_1995.PTR.raster, direction ="y")
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # CRTM05 projection is assigned to raster layer
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  crs(HTemp_GPCC_1981_1995.GPQM.raster) <- CRTM05
  crs(HTemp_PRECIS_1981_1995.GPQM.raster) <- CRTM05
  crs(HTemp_CanESM2_1981_1995.GPQM.raster) <- CRTM05
  crs(HTemp_MIROC_1981_1995.GPQM.raster) <- CRTM05
  crs(HTemp_EARTH_1981_1995.GPQM.raster) <- CRTM05 
  crs(HTemp_HadGEM2_1981_1995.GPQM.raster) <- CRTM05
  crs(HTemp_MPI_1981_1995.GPQM.raster) <- CRTM05
  crs(HTemp_ENSAMBLE_1981_1995.GPQM.raster) <- CRTM05
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  crs(HTemp_GPCC_1981_1995.DELTA.raster) <- CRTM05
  crs(HTemp_PRECIS_1981_1995.DELTA.raster) <- CRTM05
  crs(HTemp_CanESM2_1981_1995.DELTA.raster) <- CRTM05 
  crs(HTemp_MIROC_1981_1995.DELTA.raster) <- CRTM05 
  crs(HTemp_EARTH_1981_1995.DELTA.raster) <- CRTM05 
  crs(HTemp_HadGEM2_1981_1995.DELTA.raster) <- CRTM05 
  crs(HTemp_MPI_1981_1995.DELTA.raster) <- CRTM05
  crs(HTemp_ENSAMBLE_1981_1995.DELTA.raster) <- CRTM05
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  crs(HTemp_GPCC_1981_1995.PTR.raster) <- CRTM05
  crs(HTemp_PRECIS_1981_1995.PTR.raster) <- CRTM05 
  crs(HTemp_CanESM2_1981_1995.PTR.raster) <- CRTM05 
  crs(HTemp_MIROC_1981_1995.PTR.raster) <- CRTM05 
  crs(HTemp_EARTH_1981_1995.PTR.raster) <- CRTM05 
  crs(HTemp_HadGEM2_1981_1995.PTR.raster) <- CRTM05 
  crs(HTemp_MPI_1981_1995.PTR.raster) <- CRTM05
  crs(HTemp_ENSAMBLE_1981_1995.PTR.raster) <- CRTM05
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # A simple plot is requested
  plot(HTemp_GPCC_1981_1995.GPQM.raster[[1]])
  plot(HTemp_GPCC_1981_1995.DELTA.raster[[1]])
  plot(HTemp_GPCC_1981_1995.PTR.raster[[1]])
  
  #--------------------------------------------------------------------------------------------------------------------------------------
  # Rasterstack objects are exported to GeoTIFF format
  #--------------------------------------------------------------------------------------------------------------------------------------
  # GPQM
  writeRaster(HTemp_GPCC_1981_1995.GPQM.raster, paste0("GPCC_1981_1995_GPQM_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_PRECIS_1981_1995.GPQM.raster, paste0("PRECIS_1981_1995_GPQM_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_CanESM2_1981_1995.GPQM.raster, paste0("CanESM2_1981_1995_GPQM_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_MIROC_1981_1995.GPQM.raster, paste0("MIROC_1981_1995_GPQM_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_EARTH_1981_1995.GPQM.raster, paste0("EARTH_1981_1995_GPQM_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_HadGEM2_1981_1995.GPQM.raster, paste0("HadGEM2_1981_1995_GPQM_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_MPI_1981_1995.GPQM.raster, paste0("MPI_1981_1995_GPQM_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE) 
  writeRaster(HTemp_ENSAMBLE_1981_1995.GPQM.raster, paste0("ENSAMBLE_1981_1995_GPQM_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # DELTA
  #--------------------------------------------------------------------------------------------------------------------------------------
  writeRaster(HTemp_GPCC_1981_1995.DELTA.raster, paste0("GPCC_1981_1995_DELTA_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_PRECIS_1981_1995.DELTA.raster, paste0("PRECIS_1981_1995_DELTA_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_CanESM2_1981_1995.DELTA.raster, paste0("CanESM2_1981_1995_DELTA_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_MIROC_1981_1995.DELTA.raster, paste0("MIROC_1981_1995_DELTA_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_EARTH_1981_1995.DELTA.raster, paste0("EARTH_1981_1995_DELTA_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_HadGEM2_1981_1995.DELTA.raster, paste0("HadGEM2_1981_1995_DELTA_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_MPI_1981_1995.DELTA.raster, paste0("MPI_1981_1995_DELTA_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_ENSAMBLE_1981_1995.DELTA.raster, paste0("ENSAMBLE_1981_1995_DELTA_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
  # PTR
  #--------------------------------------------------------------------------------------------------------------------------------------
  writeRaster(HTemp_GPCC_1981_1995.PTR.raster, paste0("GPCC_1981_1995_PTR_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_PRECIS_1981_1995.PTR.raster, paste0("PRECIS_1981_1995_PTR_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_CanESM2_1981_1995.PTR.raster, paste0("CanESM2_1981_1995_PTR_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_MIROC_1981_1995.PTR.raster, paste0("MIROC_1981_1995_PTR_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_EARTH_1981_1995.PTR.raster, paste0("EARTH_1981_1995_PTR_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_HadGEM2_1981_1995.PTR.raster, paste0("HadGEM2_1981_1995_PTR_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_MPI_1981_1995.PTR.raster, paste0("MPI_1981_1995_PTR_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  writeRaster(HTemp_ENSAMBLE_1981_1995.PTR.raster, paste0("ENSAMBLE_1981_1995_PTR_BC_",V.select.month[i],".tif"), format="GTiff", overwrite = TRUE)
  #--------------------------------------------------------------------------------------------------------------------------------------
  
} # Main loop is closed

#--------------------------------------------------------------------------------------------------------------------------------------
# END of code
#--------------------------------------------------------------------------------------------------------------------------------------
